﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Microsoft.Data.SqlRemoteBlobs.Setup;
using System.Diagnostics;
using System.Runtime.InteropServices;
using Microsoft.Win32.SafeHandles;
using System.Text.RegularExpressions;
using System.IO;
using System.Configuration;
using System.Xml.Linq;
using System.Xml.XPath;

namespace Tests.Installer
{
    /// <summary>
    /// Summary description for ClientOnly
    /// </summary>
    [TestClass]
    public class ClientOnly
    {

        MemoryStream memoryStreamConsole;
        StreamWriter streamWritterConsole;

        protected string GetStdOut()
        {
            streamWritterConsole.Flush();

            var rval = Encoding.Default.GetString(memoryStreamConsole.ToArray());
            System.Diagnostics.Trace.WriteLine(rval);            
            return rval;
        }
        
        public ClientOnly()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        [TestInitialize()]
        public void TestInitialize()
        {
            memoryStreamConsole = new MemoryStream();
            streamWritterConsole = new StreamWriter(memoryStreamConsole);
            Console.SetOut(streamWritterConsole);            
        }

        [TestMethod]
        public void CheckParamString()
        {           
            InstallProviderSetup.Main( null);
            var stdout = GetStdOut();            
            Assert.AreEqual(true, stdout.Contains("CLIENTCONFIG"),"Config param CLIENTCONFIG not found");
        }

        [TestMethod]
        public void CheckWithParamClientConfig()
        {
            InstallProviderSetup.Main(new string[] { "-CLIENTCONFIG" });
            var stdout = GetStdOut();            
            /*
            The required switch CONFIGURATIONFILE is missing.
            The required switch NAME            is missing.
             * */
            Assert.AreEqual(true, stdout.Contains("The required switch CONFIGURATIONFILE"), "The required switch CONFIGURATIONFILE not in console out");
            Assert.AreEqual(true, stdout.Contains("The required switch NAME"), "The required switch NAME not in console out");
            // with the -CLIENTCONFIG we only expect 2 more params            
            Assert.AreEqual(2, Regex.Matches(stdout, "The required switch").Count); 
        }

        [TestMethod]
        public void CheckForMachineConfigChange_INSTALL()
        {
            var machineConfig = ConfigurationManager.OpenMachineConfiguration();
            var backupConfigFile = Path.GetFileName(machineConfig.FilePath) + ".bak";
            string registerSuccessString = "Registering the provider configuration into the .NET machine.config file.";

            // Lets backup the config
            File.Copy(machineConfig.FilePath, backupConfigFile,true);
            try
            {
                InstallProviderSetup.Main(
                    new string[] {
                        "-CLIENTCONFIG", 
                        "-configurationfile",@"..\..\..\Tests\TestConfiguration.xml",
                        "-NAME", "Test"
                    });
                
                // Make sure the message we get back says its worked.
                var stdout = GetStdOut();
                Assert.AreEqual(true, stdout.Contains(registerSuccessString), string.Format("Output from InstallProvider did not contain {0}", registerSuccessString));

                // Now check thats the XML fragment is in our machine.config
                var machineXML = XDocument.Load(machineConfig.FilePath);
                var provider = machineXML.XPathSelectElement("configuration/RemoteBlobStorage/Providers/Test");
                Assert.AreNotEqual (null, provider, "Config for providor not found in MachineConfig");
            }
            finally
            {
                if (File.Exists(backupConfigFile))
                {
                    File.Copy(Path.GetFileName(machineConfig.FilePath) + ".bak", machineConfig.FilePath, true);
                }
            }
        }

        [TestMethod]
        public void CheckForMachineConfigChange_UNINSTALL()
        {
            CheckForMachineConfigChange_INSTALL();

            var machineConfig = ConfigurationManager.OpenMachineConfiguration();
            var backupConfigFile = Path.GetFileName(machineConfig.FilePath) + ".bak";
            string registerSuccessString = "Registering the provider configuration into the .NET machine.config file.";

            // Lets backup the config
            File.Copy(machineConfig.FilePath, backupConfigFile,true);
            try
            {
                InstallProviderSetup.Main(
                    new string[] {
                        "-UNINSTALL",
                        "-CLIENTCONFIG", 
                        //"-configurationfile",@"..\..\..\Tests\TestConfiguration.xml",
                        "-NAME", "Test"
                    });

                // Make sure the message we get back says its worked.
                var stdout = GetStdOut();
                Assert.AreEqual(true, stdout.Contains(registerSuccessString), string.Format("Output from InstallProvider did not contain {0}", registerSuccessString));

                // Now check thats the XML fragment is in our machine.config
                var machineXML = XDocument.Load(machineConfig.FilePath);
                var provider = machineXML.XPathSelectElement("configuration/RemoteBlobStorage/Providers/Test");
                Assert.AreEqual(null, provider, "Config for providor not found in MachineConfig");
            }
            finally
            {
                if (File.Exists(backupConfigFile))
                {
                    File.Copy(Path.GetFileName(machineConfig.FilePath) + ".bak", machineConfig.FilePath, true);
                }
            }
        }


    } 
}
